#!/bin/bash

SCRIPT=autoexec.sh
FILE=/mnt/${1}/${SCRIPT}
PIDFILE="/var/run/autorun_process.pid"
LOCKFILE="/tmp/__autorun_lock__"
POPUPIDFILE="/tmp/.autorun_msg_id"

function wait_confirm() {

    # Wait for boot to complete ( SS running )
    while [ ! -e /var/run/jmuconfig.pid ]; do
        sleep 1
    done

    ID=$( DISPLAY=:0 dbus-send --print-reply --session --dest=com.exor.ShowMessage "/" com.exor.ShowMessage.showMessageAsync \
            string:"An autorun script from an external storage has been found.<br>Do you want to execute it?<br>" \
            array:string:"Yes","Cancel" string:"USB Autorun" 2>/dev/null | grep '/ShowMessageInstance/' | sed 's:.*/ShowMessageInstance/\([0-9]\+\)":\1:' )

    [ -z "$ID" ] && exit 0

    echo "$ID:$DEVNAME" > $POPUPIDFILE

    while read lline; do

        echo $lline

        if [ -n "$done" ]; then
            res="$( echo $lline | awk '{ print $2}' )"
            pkill -P $!
            break
        fi

        if ( echo $lline | grep -q "path=/ShowMessageInstance/$ID" ); then
            done=1
            continue
        fi

    done < <( DISPLAY=:0 dbus-monitor --session type='signal',interface=com.exor.ShowMessageInstance,member=done )

    rm $POPUPIDFILE
    pkill -P $$

    [ "$res" != "1" ] && exit 0
}

function close_popup() {
    DISPLAY=:0 dbus-send --print-reply --session --dest=com.exor.ShowMessage "/ShowMessageInstance/$1" com.exor.ShowMessageInstance.close
}

autorun() {
    # wait until the system has boot (no rc scripts running up to 20 seconds)
    for i in `seq 1 20` ; do
        XX="`ps aux`" ; if  ! ( echo $XX | grep "/rc " ) ; then
		break;
	else
		sleep 1
		cat /proc/mounts | grep "$DEVNAME " | grep "/mnt/$1 " || exit
		echo "Waiting system boot complete before starting $SCRIPT script $1 ..." | logger -t "AUTORUN"
	fi
    done

    if [ -e /mnt/${1}/resetnetworksettings ]; then
        echo "Network reset requested - fowarding to EPAD" | logger -t "AUTORUN"
        dbus-send --print-reply --system --dest=com.exor.EPAD "/NetworkManager" com.exor.EPAD.NetworkManager.resetConfiguration

        # continuous beep to force user to remove key and reboot
        dbus-send --system --print-reply --dest=com.exor.EPAD "/Buzzer" com.exor.EPAD.Buzzer.beep int32:440 int32:-1
    fi

    # exit if autoexec is locked by factory setting
    [ "$(/usr/bin/sys_params -r factory/services/autorun/mode)" = "locked" ] && exit

    if [ -f ${FILE} ]; then

	# BSP-2890 Detect if it's a USBUpdater
	if [ -e "/mnt/${1}/src/updateos.img" ]; then
		# Version is expected somewhere at the beginning of the file.
		# Minimum version allowed is 1.33
		version="$( cat ${FILE} | head -n15 | sed -n 's:^VERSION="\(.*\)\s*":\1:p' )"
		minVers="$( echo -e "1.33\n${version}" | sort -n -t . -k 1 | head -n1 )"

		if [ -z "$version" -o "$minVers" != "1.33" ]; then
			if ! (pidof psplash &>/dev/null ); then
				psplash --notouch --no-progress-bar &
				sleep 1
				psplash-write "MSG Unsupported USBUpdater version detected
Aborting...

"
				sleep 8
				psplash-write "QUIT"
			fi
			exit 0
		fi
	fi

        # Show popup before executing
        [ "$(/usr/bin/sys_params -r factory/services/autorun/notify)" = "true" ] && wait_confirm

        /bin/bash ${FILE}
    fi
}

if [ "$ACTION" = "remove" ]; then

    if [ -e "$POPUPIDFILE" ]; then
        ID=$( cat $POPUPIDFILE | cut -d':' -f1 )
        DEV=$( cat $POPUPIDFILE | cut -d':' -f2 )

        [ $DEVNAME = "$DEV" ] && close_popup $ID
        rm $POPUPIDFILE
    fi

    exit 0
fi

if [ "${@: -1}" = "start" ]; then
        autorun $@
        exit 0
fi

# consistecy check : are we executing the script from the deviced signalled by kernel?
cat /proc/mounts | grep "$DEVNAME " | grep "/mnt/$1 " || exit

(
        flock -w 1 200 || exit

        # consistency check: avoid running multiple scripts in parallel
        [ -e "$PIDFILE" -a -d "/proc/$( head -n1 "$PIDFILE" )" ] && exit
        [ -e "$PIDFILE" ] && rm "$PIDFILE"

        /usr/sbin/start-stop-daemon -S --background --startas /bin/sh --pidfile $PIDFILE --make-pidfile -- -c "/etc/udev/scripts/usb_autorun.sh $@ start"

) 200>>$LOCKFILE
